package bin;

import kernel.ps.ProcessHandle;
import kernel.ps.Process;
import kernel.peripherals.Peripherals.Peripheral;

using tink.CoreApi;
using Lambda;

class Disk implements Process {
	private var handle:ProcessHandle;

	public function new() {}

	public function run(handle:ProcessHandle):Void {
		this.handle = handle;
		var subcommand = handle.args[0];
		var driveAddr:Null<String> = handle.args[1];

		switch (subcommand) {
			case "ls":
				Peripheral.instance.getDrives().foreach(drive -> {
					var addr = drive.getAddr();
					var label = drive.getDiskLabel();
					var id = drive.getDiskID();

					if (drive.isDiskPresent()){
						if (drive.hasAudio()){
							handle.writeLine('${addr} => ${label} [AUDIO]');
						}else{
							handle.writeLine('${addr} => ${label} (${id})');
						}
					}else {
						handle.writeLine('${addr} => [NO DISK]');
					}

					return true;
				});
			case "play":
				var drive = Peripheral.instance.getDrive(driveAddr);

				if (drive == null){
					handle.writeLine("Drive not found: " + driveAddr);
				}

				if (!drive.isDiskPresent()){
					handle.writeLine("No disk in drive: " + driveAddr);
				}

				if (!drive.hasAudio()){
					handle.writeLine("Disk in drive " + driveAddr + " does not have audio");
				}

				drive.playAudio();
			case "stop":
				var drive = Peripheral.instance.getDrive(driveAddr);

				if (drive == null){
					handle.writeLine("Drive not found: " + driveAddr);
				}

				if (!drive.isDiskPresent()){
					handle.writeLine("No disk in drive: " + driveAddr);
				}

				if (!drive.hasAudio()){
					handle.writeLine("Disk in drive: " + driveAddr + " does not have audio");
				}

				drive.stopAudio();

			case "eject":
				var drive = Peripheral.instance.getDrive(driveAddr);

				if (drive == null){
					handle.writeLine("Drive not found: " + driveAddr);
				}

				if (!drive.isDiskPresent()){
					handle.writeLine("No disk in drive: " + driveAddr);
				}

				drive.ejectDisk();
			case "lable":
				var drive = Peripheral.instance.getDrive(driveAddr);
				var label:String = handle.args[2];

				if (drive == null){
					handle.writeLine("Drive not found: " + driveAddr);
				}

				if (!drive.isDiskPresent()){
					handle.writeLine("No disk in drive: " + driveAddr);
				}

				if (label == null || label == ""){
					handle.writeLine(drive.getDiskLabel());
				}else{
					var err  = drive.setDiskLabel(label);
					if (err != null){
						handle.writeLine("Failed to set lable");
					}
				}
			case "help":
			case null:
				printHelp();
			default:
				handle.writeLine("Unknown subcommand: " + subcommand);
				printHelp();
		}

		return handle.close(true);
	}

	private function printHelp() {
		handle.writeLine("Usage: disk <subcommand> [args]");
		handle.writeLine("Subcommands:");
		handle.writeLine("  ls");
		handle.writeLine("  play <drive>");
		handle.writeLine("  stop <drive>");
		handle.writeLine("  eject <drive>");
		handle.writeLine("  label <drive> [label]");
	}
}
